@@ -3,18 +3,14 @@ use strict;
 use warnings;
 use base 'Tie::Array';
 use Storable ();
-
-# create new tied array
-sub TIEARRAY {
-    my ($class, $share) = @_;
-    my $self = bless { share => $share }, $class;
-    $self;
-}
-
+use IPC::ShareLite ':lock';
 
 sub _get {
     my $self = shift;
-    return $self->{share}->get('array');
+
+    return $self->{share}->lock_cb(sub {
+        $self->{share}->get('array');
+    }, LOCK_SH);
 }
 sub FETCH {
     my ($self, $index) = @_;
@@ -26,15 +22,21 @@ sub FETCHSIZE {
     scalar @$ary;
 }
 
+sub TIEARRAY {
+    my ($class, $share) = @_;
+    my $self = bless { share => $share }, $class;
+    $self;
+}
+
 sub STORE {
     my ($self, $index, $val) = @_;
 
     $self->{share}->lock_cb(sub {
         my $share = $self->{share};
-        my $cur = $share->get_nolock('array');
+        my $cur = $share->get('array');
         $cur->[$index] = $val;
-        $share->set_nolock(array => $cur);
-    });
+        $share->set(array => $cur);
+    }, LOCK_EX);
 }
 
 1;
